#!/bin/bash

# I want my default user to be nobody.  you should be able to configure it to be any user (including root)
USER_NAME=${USER_NAME:-nobody}
USER_UID=${USER_UID}
USER_GID=${USER_GID}

COMMAND="$@"

# Check to see if we have been passed in a user name other than "root".  If we have, create it
#  if it does not exist and then switch to it.
if [[ -n "${USER_NAME}" && "${USER_NAME}" != "root" ]]; then
    (getent passwd ${USER_NAME} > /dev/null) || USER_DOESNT_EXIST=1

    # If the user does not exist, create it now.
    if [[ ${USER_DOESNT_EXIST} ]]; then
        if [ -n "${USER_UID}" ]; then
            UID_STR="--uid ${USER_UID}"
        fi
        if [ -n "${USER_GID}" ]; then
            # Fail silently if this already exists
            groupadd -f -g ${USER_GID} "${USER_NAME}_g" 2>/dev/null
            GID_STR="--gid ${USER_GID}"
        fi
        # Create and set correct permissions on the temp home folder
        useradd --shell /bin/bash ${UID_STR} ${GID_STR} --non-unique --comment "" -M ${USER_NAME}
    fi

    # Our working directory is "/root".  I am going to cheat a bit here and set the home
    #  directory for the user we are about o become to root.  Then we change the ownership
    #  of everything to this user. (I don't want to chown /root/.conan/data since this is
    #  mount point..)
    usermod -d $(pwd) ${USER_NAME} >/dev/null 2>&1
    gosu root chmod +w $(pwd)
    gosu root find . -path ./.conan/data -prune -or -print | xargs -I@ gosu root chown ${USER_NAME} @
    gosu root chown ${USER_NAME} .conan

elif [ -z "${USER_NAME}" ]; then
    USER_NAME=root
fi

if [ -e "/var/run/docker.sock" ]; then
    DOCKER_GID=$(stat -c "%g" /var/run/docker.sock)
    OLD_GROUP=$(getent group ${DOCKER_GID} | cut -d ':' -f 1)
    if [[ $OLD_GROUP -ne 0 ]]; then
        if [[ -n "$OLD_GROUP" && "$OLD_GROUP" != "docker_g" ]]; then
            echo "*** docker group id ${DOCKER_GID} is already taken by $OLD_GROUP"
            echo "*** change docker group id on the host system to resolve"
            echo "*** do this ya lazy:"
            echo "### https://unix.stackexchange.com/questions/33844/change-gid-of-a-specific-group"
        fi
        gosu root groupadd -f -g ${DOCKER_GID} "docker_g" 2>/dev/null
        gosu root usermod -a -G docker_g $USER_NAME 2>/dev/null
    else
        # This logic is specifically for Docker Desktop
        #  It uses a private VM, so we do not have control of the docker socket from the host. In this case we detect that the socket
        #  is oweed by root.  We add ourself to the root group and add group write privelages to the socket.  This allows us to use
        #  docker-in-docker.
        #
        #  Note this does change the underlying VM, which could be a security risk for other containers.
        gosu root usermod -a -G root $USER_NAME 2>/dev/null
        gosu root chmod g+w /var/run/docker.sock 2>/dev/null
    fi
fi

if [[ -x "$(command -v conan)" && -n "${CONAN_REMOTE}" ]]; then
    gosu ${USER_NAME} conan remote add ${CONAN_REMOTE_NAME} ${CONAN_REMOTE} &>/dev/null
fi

# Run the command as the requested user
exec -l -a ${USER_NAME} gosu ${USER_NAME} bash -lc "$COMMAND"
